/*
 * blamp.c
 *
 * Created: 12.05.2017 13:14:00
 *  Author: dsosnowski
 */ 


#define F_CPU 1000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "wdog.h"

//------------------------------------------------------------------
#define OUT_PIN				0		//H
#define PWM_TIM				TIMER0A
#define LED_PIN				1		//H
#define REG_PIN				4		//A
#define REG_CH				2		//ADC
#define SW_PIN				2

#define MAIN_DELAY			5

#define PWM_RESOLUTION		255
#define PWM_START_VALUE		0
#define PWM_STOP_VALUE		255 //(PWM_RESOLUTION - 2)
#define PWM_RISE_DELAY		2
#define PWM_FALL_DELAY		10
#define SYSTEM_MIN_TIME		15

#define LED_DUTY			10
#define LED_PERIOD			100

#define OUT_SET				(PORTB |= (1<<OUT_PIN))
#define OUT_OFF				(PORTB &= ~(1<<OUT_PIN))
#define	LED_SET				(PORTB |= (1<<LED_PIN))
#define	LED_OFF				(PORTB &= ~(1<<LED_PIN))

#define SW_GET				(PINB & (1<<SW_PIN))
#define SW_HOLD_THR			250
#define UI_ACTION_CLICK		1
#define UI_ACTION_HOLD		2

#define SYSTEM_STANDBY		0
#define SYSTEM_RISE			1
#define SYSTEM_TIME			2
#define SYSTEM_FALL			3

void ActionMan(uint8_t state);
void PwmProc(void);
void AnalogProc(void);
uint16_t AnalogGet(void);
void UiSwitchProc(void);
uint8_t UiSwitchGet(void);
void UiLedProc(void);
void HardInit(void);
void AnalogInit(void);
void PwmInit(void);
void PwmEnable(void);
void PwmDisable(void);


//------------------------------------------------------------------
struct {
	uint8_t duty;
	uint8_t delay;
} pwm;

struct {
	uint8_t led;
	uint8_t action;
	uint16_t reg;
} ui;

struct {
	uint8_t step;
	uint8_t pwr;
	uint16_t time;
	uint8_t delay;
} system;


//------------------------------------------------------------------
int main(void)
{
	uint16_t persec = 0;
	
	
	HardInit();
	AnalogInit();
	PwmInit();
	
	//system.step = SYSTEM_RISE;
	ActionMan(SYSTEM_RISE);
	
	while(1)
	{
		
		AnalogProc();
		UiSwitchProc();
		
		//---------------------------------------------
		if ((UiSwitchGet() & UI_ACTION_CLICK) > 0) {
			if (system.step == SYSTEM_STANDBY){
				system.step = SYSTEM_RISE;
			} else if (system.step == SYSTEM_FALL){
				system.step = SYSTEM_RISE;
			} else {
				system.step = SYSTEM_TIME;				
			}
			ActionMan(system.step);
		}
		
		if ((UiSwitchGet() & UI_ACTION_HOLD) > 0) {
			ActionMan(SYSTEM_FALL);	
		}

		//---------------------------------------------
		if (system.step == SYSTEM_TIME){
			persec++;
			if (persec >= (1000 / MAIN_DELAY)){
				persec = 0;
				system.time++;
				if (system.time >= AnalogGet()) {
					system.time = 0;
					//system.step = SYSTEM_FALL;
					ActionMan(SYSTEM_FALL);			
				}
				_delay_ms(5);
			}
		} else {
			persec = 0;
		}


		//---------------------------------------------
		if (pwm.delay < system.delay){
			pwm.delay++;
		} else {
			pwm.delay = 0;
			PwmProc();
		}
		
		UiLedProc();		
		
		_delay_ms(MAIN_DELAY);
		WdtReset();
	}
}

//------------------------------------------------------------------
void ActionMan(uint8_t state){
	system.step = state;
	switch (state){
		case SYSTEM_STANDBY:{
			system.delay = 0;
			system.pwr = 0;
			ui.led = 0;
		} break;
		case SYSTEM_RISE:{
			system.delay = PWM_RISE_DELAY;
			system.pwr = 1;
			ui.led = 1;
		} break;
		case SYSTEM_TIME:{
			system.delay = 0;
			system.pwr = 1;
			system.time = 0;
			ui.led = 2;
		} break;
		case SYSTEM_FALL:{
			system.delay = PWM_FALL_DELAY;
			system.pwr = 0;
			ui.led = 1;			
		} break;
	}
}

//------------------------------------------------------------------
void PwmProc(void){	
	if (system.pwr > 0){
		if (pwm.duty < PWM_STOP_VALUE) {
			pwm.duty++;			
			OCR0A = (0xFF - pwm.duty);
			OUT_OFF;
			PwmEnable();

			if (pwm.duty == PWM_STOP_VALUE) {
				ActionMan(SYSTEM_TIME);
			}
		} else {
			OUT_SET;
			PwmDisable();
			pwm.duty = 0xFF;
		}		
	} else {
		if (pwm.duty > PWM_START_VALUE) {
			pwm.duty--;
			OCR0A = (0xFF - pwm.duty);
			OUT_OFF;
			PwmEnable();

			if (pwm.duty == PWM_START_VALUE) {
				ActionMan(SYSTEM_STANDBY);
			}
		} else {
			OUT_OFF;
			PwmDisable();
			pwm.duty = 0;
			
			if (system.step != SYSTEM_STANDBY){
				ActionMan(SYSTEM_STANDBY);
			}
		}		
	}	
}

//------------------------------------------------------------------
void AnalogProc(void){
	ui.reg = ADCH;	
	ui.reg <<= 1;
	ui.reg += SYSTEM_MIN_TIME;
}

//------------------------------------------------------------------
uint16_t AnalogGet(void){
	return ui.reg;
}

//------------------------------------------------------------------
void UiSwitchProc(void){
	static uint8_t hold;
	
	ui.action = 0;
	
	if (SW_GET == 0){
		if (hold < 0xFF) {
			hold++;
		}
		if (hold == SW_HOLD_THR) {
			ui.action += UI_ACTION_HOLD;			
		}
	} else {
		if ((hold > 2) && (hold < SW_HOLD_THR)){
			ui.action += UI_ACTION_CLICK;
		}
		hold = 0;
	}
}

//------------------------------------------------------------------
uint8_t UiSwitchGet(void){
	return ui.action;
}

//------------------------------------------------------------------
void UiLedProc(void){
	static uint16_t duty;
	
	duty++;

	switch (ui.led) {
		case 0: {
			if (duty >= 800){
				duty = 0;
			}

			if (duty < LED_DUTY){
				LED_SET;
			} else {
				LED_OFF;
			}			
		} break;
		case 1:{
			if (duty >= LED_PERIOD){
				duty = 0;
			}

			if (duty < LED_DUTY){
				LED_SET;
			} else {
				LED_OFF;
			}			
		} break;
		case 2: {
			LED_SET;
		}
	}
}

//------------------------------------------------------------------
void HardInit(void){
	PORTB = (1<<SW_PIN);
	DDRB = (1<<LED_PIN | 1<<OUT_PIN);
	ACSR = (1<<ACD);
	PRR = (1<<PRTIM1 | 1<<PRUSI);
	WdtSet(7);
}

//----------------------------------
void AnalogInit(void){
	DIDR0 = (1<<ADC2D);
	ADCSRB = 0;
	ADCSRA = (1<<ADEN | 1<<ADATE | 1<<ADPS2 | 1<<ADPS1);
	ADMUX = (1<<ADLAR | REG_CH);
	ADCSRA |= (1<<ADSC);
}

//----------------------------------
void PwmInit(void){
	TCCR0A = (1<<WGM01 | 1<<WGM00);		//fast pwm
	TCCR0B = (1<<CS01);
}

//----------------------------------
void PwmEnable(void){
	TCCR0A = (1<<WGM01 | 1<<WGM00 | 1<<COM0A1 | 1<<COM0A0);
}

//----------------------------------
void PwmDisable(void){
	TCCR0A = (1<<WGM01 | 1<<WGM00);
}
